%-----------------------------------------------------------------------------%
% vim: ft=prolog ts=4 sw=4 et wm=0 tw=0
%-----------------------------------------------------------------------------%
:- use_module(turing).


%-----------------------------------------------------------------------------%
% Sample Turing machines.
%-----------------------------------------------------------------------------%

% A tape with a string of 1 will have a 1 appended.
% Input tape: [1, 1, 1]
% Output tape: [1, 1, 1, 1]
% Uses the default configuration from the turing module.
concatenative_increment(q0, 1, 1, right, q0).
concatenative_increment(q0, b, 1, stay,  qf).

% A three-state busy beaver
% http://en.wikipedia.org/wiki/Busy_beaver
bb_config(IS, FS, RS, B, S) :-
	IS = 'A',               % initial state
	FS = ['HALT'],          % halting states
	RS = [IS, 'B', 'C'],    % running states
	B  = 0,                 % blank symbol
	S  = [B, 1].            % valid symbols
busy_beaver('A', 0, 1, right, 'B').
busy_beaver('A', 1, 1, left,  'C').
busy_beaver('B', 0, 1, left,  'A').
busy_beaver('B', 1, 1, right, 'B').
busy_beaver('C', 0, 1, left,  'B').
busy_beaver('C', 1, 1, stay,  'HALT').


go :-
	turing(default_config, concatenative_increment, [1, 1, 1], T1),
	write(T1), nl,
	turing(bb_config, busy_beaver, [], T2),
	write(T2), nl.
